@@ -1,4 +1,3 @@
-import os
 import dbus
 import logging
 from dbus.mainloop.glib import DBusGMainLoop
@@ -15,7 +14,7 @@ class UDisksBackend(Backend):
     def __init__(self, allow_system_internal=False, bus=None, show_all=False):
         Backend.__init__(self)
         self.mounted_source = ''
-        self.formatting = None
+        self.formatting = []
         self.show_all = show_all
         self.allow_system_internal = allow_system_internal
         logging.debug('UDisksBackend')
@@ -44,7 +43,7 @@ class UDisksBackend(Backend):
                 break
             try:
                 device_i.FilesystemMount('', [])
-            except dbus.DBusException, e:
+            except dbus.DBusException as e:
                 if e._dbus_error_name != 'org.freedesktop.UDisks.Error.Busy':
                     raise
                 logging.debug('Busy.')
@@ -114,7 +113,7 @@ class UDisksBackend(Backend):
         if not get('device-is-mounted'):
             try:
                 mp = dk.FilesystemMount('', [], dbus_interface=DEVICE_IFACE)
-            except dbus.DBusException, e:
+            except dbus.DBusException as e:
                 logging.exception('Could not mount the device:')
                 return
         mount = get('device-mount-paths')[0]
@@ -135,6 +134,8 @@ class UDisksBackend(Backend):
         def get(prop):
             return dk.Get(device, prop, dbus_interface=PROPS_IFACE)
 
+        model = get('DriveModel')
+        vendor = get('DriveVendor')
         fstype = get('id-type')
         logging.debug('id-type: %s' % fstype)
         if fstype == 'vfat':
@@ -147,7 +148,7 @@ class UDisksBackend(Backend):
         if fstype == 'vfat' and not get('device-is-mounted'):
             parent_i = self.bus.get_object(DISKS_IFACE, parent)
             parent_f = parent_i.Get(parent, 'device-file', dbus_interface=PROPS_IFACE)
-            if self.formatting != parent_f:
+            if device not in self.formatting and parent not in self.formatting:
                 try:
                     self.retry_mount(device)
                 except:
@@ -167,14 +168,17 @@ class UDisksBackend(Backend):
         device_file = get('device-file')
         if total > 0:
             self.targets[unicode(device)] = {
-                'label' : unicode(label),
-                'free' : free,
-                'device' : unicode(device_file),
-                'capacity' : total,
-                'status' : status,
+                'vendor'     : vendor,
+                'model'      : model,
+                'label'      : unicode(label),
+                'free'       : free,
+                'device'     : unicode(device_file),
+                'capacity'   : total,
+                'status'     : status,
                 'mountpoint' : mount,
-                'persist' : 0,
-                'parent' : unicode(parent),
+                'persist'    : 0,
+                'parent'     : unicode(parent),
+                'formatting' : False,
             }
             self._update_free(unicode(device))
             if self.show_all:
@@ -195,18 +199,23 @@ class UDisksBackend(Backend):
         dk = self.bus.get_object(DISKS_IFACE, device)
         def get(prop):
             return dk.Get(device, prop, dbus_interface=PROPS_IFACE)
+        model = get('DriveModel')
+        vendor = get('DriveVendor')
         device_file = get('device-file')
         size = get('device-size')
         if size > 0:
             self.targets[unicode(device)] = {
-                'label' : '',
-                'free' : -1,
-                'device' : unicode(device_file),
-                'capacity' : size,
-                'status' : NEED_FORMAT,
+                'vendor'     : vendor,
+                'model'      : model,
+                'label'      : '',
+                'free'       : -1,
+                'device'     : unicode(device_file),
+                'capacity'   : size,
+                'status'     : NEED_FORMAT,
                 'mountpoint' : None,
-                'persist' : 0,
-                'parent' : None,
+                'persist'    : 0,
+                'parent'     : None,
+                'formatting' : False,
             }
             if callable(self.target_added_cb):
                 if self.show_all:
@@ -259,18 +268,33 @@ class UDisksBackend(Backend):
             return ''
         return mp
     
+    def format_done(self, dev=None):
+        if dev in self.targets:
+            p = self.targets[dev]['parent']
+            if p and p in self.targets:
+                dev = p
+            self.targets[dev]['formatting'] = False
+            self.formatting.remove(dev)
+
+    def format_failed(self, message, dev=None):
+        self.format_done(dev)
+        self.format_failed_cb(message)
+
     def format(self, device):
         try:
             dk = self.bus.get_object(DISKS_IFACE, device)
-            device = dk.Get(device, 'device-file', dbus_interface=PROPS_IFACE)
-            if dk.Get(device, 'device-is-partition', dbus_interface=PROPS_IFACE):
-                device = dk.Get(device, 'partition-slave', dbus_interface=PROPS_IFACE)
-                dk = self.bus.get_object(DISKS_IFACE, device)
-                device = dk.Get(device, 'device-file', dbus_interface=PROPS_IFACE)
-            self.formatting = device
-            self.helper.Format(device, self.allow_system_internal,
-                               reply_handler=self.format_ended_cb,
-                               error_handler=self.format_failed_cb)
+            dev = dk.Get(device, 'device-file', dbus_interface=PROPS_IFACE)
+            p = self.targets[device]['parent']
+            if p and p in self.targets:
+                self.formatting.append(p)
+                self.targets[p]['formatting'] = True
+            else:
+                self.formatting.append(device)
+                self.targets[device]['formatting'] = True
+            self.helper.Format(dev, self.allow_system_internal,
+                    # There must be a better way...
+                    reply_handler=lambda: self.format_done(device),
+                    error_handler=lambda x: self.format_failed(x, device))
         except dbus.DBusException:
             # Could not talk to usb-creator-helper or devkit.
             logging.exception('Could not format the device:')
@@ -339,6 +363,8 @@ class UDisksBackend(Backend):
         except:
             # TODO let the user know via the frontend.
             logging.exception('Could not unmount the source ISO.')
+
+    def shutdown(self):
         try:
             self.helper.Shutdown()
         except dbus.DBusException:
