@@ -21,10 +21,11 @@
 Python wrappers around win32 widgets and window classes
 '''
 
-from defs import *
 import sys
 import os
 
+from usbcreator.frontends.winui.winui.defs import *
+
 __all__ = ["Window", "Frontend"]
 
 #TBD use weakref in _event_handlers_
